@@ -1,26 +1,33 @@
-import unittest2 as unittest
+# -*- coding: utf-8 -*-
 from plone.testing.zca import UNIT_TESTING
-
-import os
-import tempfile
-
-# Load patch for Zope 2.10
-try:
-    from ZPublisher import interfaces
-except ImportError:
-    import ZPublisherEventsBackport
-
-
-from zope.interface import Interface, implements, alsoProvides
-from zope.component import adapts, provideAdapter, provideUtility
-
-from plone.transformchain.interfaces import ITransform, ITransformer
+from plone.transformchain.interfaces import ITransform
+from plone.transformchain.interfaces import ITransformer
 from plone.transformchain.transformer import Transformer
 from plone.transformchain.zpublisher import applyTransformOnSuccess
-
+from zope.component import adapter
+from zope.component import provideAdapter
+from zope.component import provideUtility
+from zope.interface import alsoProvides
+from zope.interface import implementer
+from zope.interface import Interface
 from ZPublisher.HTTPResponse import default_encoding
 from ZPublisher.Iterators import filestream_iterator
-from ZServer.FTPRequest import FTPRequest
+
+import os
+import pkg_resources
+import six
+import tempfile
+import unittest
+
+
+HAS_ZSERVER = True
+try:
+    dist = pkg_resources.get_distribution('ZServer')
+except pkg_resources.DistributionNotFound:
+    HAS_ZSERVER = False
+
+if HAS_ZSERVER:
+    from ZServer.FTPRequest import FTPRequest
 
 
 class FauxPubEvent(object):
@@ -28,12 +35,15 @@ class FauxPubEvent(object):
     def __init__(self, request):
         self.request = request
 
+
 class IRequestMarker(Interface):
     pass
 
+
 class IPublishedMarker(Interface):
     pass
 
+
 class FauxResponse(object):
 
     def __init__(self, body=''):
@@ -42,9 +52,11 @@ class FauxResponse(object):
 
     def getBody(self):
         return self._body
+
     def setBody(self, body):
         self._body = body
 
+
 class FauxRequest(dict):
 
     def __init__(self, published, response=None):
@@ -55,12 +67,36 @@ class FauxRequest(dict):
         self.response = response
         self.environ = {}
 
-class FauxFTPRequest(FauxRequest, FTPRequest):
-    pass
+
+if HAS_ZSERVER:
+    class FauxFTPRequest(FauxRequest, FTPRequest):
+        pass
+
 
 class FauxPublished(object):
     pass
 
+
+@implementer(ITransform)
+@adapter(Interface, Interface)
+class FauxTransformBase(object):
+
+    order = 0
+
+    def __init__(self, published, request):
+        self.published = published
+        self.request = request
+
+    def transformBytes(self, result, encoding):
+        return None
+
+    def transformUnicode(self, result, encoding):
+        return None
+
+    def transformIterable(self, result, encoding):
+        return None
+
+
 class TestTransformChain(unittest.TestCase):
 
     layer = UNIT_TESTING
@@ -70,568 +106,382 @@ class TestTransformChain(unittest.TestCase):
 
     def test_simple(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
-
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
+        class Transform1(FauxTransformBase):
 
             def transformBytes(self, result, encoding):
-                return result + " transformed"
+                return result + b' transformed'
 
             def transformUnicode(self, result, encoding):
-                return result + u" transformed"
+                return result + u' transformed'
 
             def transformIterable(self, result, encoding):
                 return ''.join(result) + ' transformed'
 
-        provideAdapter(Transform1, name=u"test.one")
+        provideAdapter(Transform1, name=u'test.one')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Blah"]
+        result = ['Blah']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Blah transformed", new_result)
+        self.assertEqual('Blah transformed', new_result)
 
     def test_off_switch(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform1(FauxTransformBase):
 
             order = 0
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " transformed"
+                return result + b' transformed'
 
             def transformUnicode(self, result, encoding):
-                return result + u" transformed"
+                return result + u' transformed'
 
             def transformIterable(self, result, encoding):
                 return ''.join(result) + ' transformed'
 
-        provideAdapter(Transform1, name=u"test.one")
+        provideAdapter(Transform1, name=u'test.one')
 
         published = FauxPublished()
         request = FauxRequest(published)
         request.environ['plone.transformchain.disable'] = True
 
-        result = ["Blah"]
+        result = ['Blah']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals(None, new_result)
+        self.assertEqual(None, new_result)
 
+    @unittest.skipUnless(HAS_ZSERVER, 'ZServer is optional')
     def test_ftp_request_not_transformed(self):
         request = FauxFTPRequest(FauxPublished())
-        result = ["Blah"]
+        result = ['Blah']
         new_result = self.t(request, result, 'utf8')
-        self.assertEquals(None, new_result)
+        self.assertEqual(None, new_result)
 
-    def test_transform_string(self):
+    def test_transform_bytes(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
-
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
+        class Transform1(FauxTransformBase):
 
             def transformBytes(self, result, encoding):
-                return result + " One"
+                return result + b' One'
 
-            def transformUnicode(self, result, encoding):
-                return None
-
-            def transformIterable(self, result, encoding):
-                return None
-
-        provideAdapter(Transform1, name=u"test.one")
+        provideAdapter(Transform1, name=u'test.one')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = "Blah"
+        result = b'Blah'
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Blah One", new_result)
+        self.assertEqual(b'Blah One', new_result)
 
-    def test_transform_unicode(self):
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+    def test_transform_text(self):
 
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
-            def transformBytes(self, result, encoding):
-                return None
+        class Transform1(FauxTransformBase):
 
             def transformUnicode(self, result, encoding):
-                return result + u" One"
+                return result + u' One'
 
-            def transformIterable(self, result, encoding):
-                return None
-
-        provideAdapter(Transform1, name=u"test.one")
+        provideAdapter(Transform1, name=u'test.one')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = u"Blah"
+        result = u'Blah'
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals(u"Blah One", new_result)
+        self.assertEqual(u'Blah One', new_result)
 
     def test_transform_iterable(self):
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
 
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
-            def transformBytes(self, result, encoding):
-                return None
-
-            def transformUnicode(self, result, encoding):
-                return None
+        class Transform1(FauxTransformBase):
 
             def transformIterable(self, result, encoding):
-                return result + [" One"]
+                return result + [' One']
 
-        provideAdapter(Transform1, name=u"test.one")
+        provideAdapter(Transform1, name=u'test.one')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Blah"]
+        result = ['Blah']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals(["Blah", " One"], new_result)
+        self.assertEqual(['Blah', ' One'], new_result)
 
     def test_transform_mixed(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
-
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
-            def transformBytes(self, result, encoding):
-                return None
-
-            def transformUnicode(self, result, encoding):
-                return None
+        class Transform1(FauxTransformBase):
 
             def transformIterable(self, result, encoding):
-                return u''.join(result) + u" One"
+                return u''.join(result) + u' One'
 
-        class Transform2(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform2(FauxTransformBase):
 
             order = 1
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
-            def transformBytes(self, result, encoding):
-                return None
-
             def transformUnicode(self, result, encoding):
-                return result.encode(encoding) + ' Two'
+                return result.encode(encoding) + u' Two'.encode(encoding)
 
-            def transformIterable(self, result, encoding):
-                return None
-
-        class Transform3(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform3(FauxTransformBase):
 
             order = 2
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result.decode(encoding) + u" Three"
+                return result.decode(encoding) + u' Three'
 
-            def transformUnicode(self, result, encoding):
-                None
-
-            def transformIterable(self, result, encoding):
-                return None
-
-        provideAdapter(Transform1, name=u"test.one")
-        provideAdapter(Transform2, name=u"test.two")
-        provideAdapter(Transform3, name=u"test.three")
+        provideAdapter(Transform1, name=u'test.one')
+        provideAdapter(Transform2, name=u'test.two')
+        provideAdapter(Transform3, name=u'test.three')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Blah"]
+        result = ['Blah']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals(u"Blah One Two Three", new_result)
+        self.assertEqual(u'Blah One Two Three', new_result)
 
     def test_abort(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform1(FauxTransformBase):
+            pass
 
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
-            def transformBytes(self, result, encoding):
-                return None
-
-            def transformUnicode(self, result, encoding):
-                return None
-
-            def transformIterable(self, result, encoding):
-                return None
-
-        provideAdapter(Transform1, name=u"test.one")
+        provideAdapter(Transform1, name=u'test.one')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Blah"]
+        result = ['Blah']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals(["Blah"], new_result)
+        self.assertEqual(['Blah'], new_result)
 
     def test_abort_chain(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
-
-            order = 0
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
+        class Transform1(FauxTransformBase):
 
             def transformBytes(self, result, encoding):
-                return "One"
+                return b'One'
 
             def transformUnicode(self, result, encoding):
-                return "One"
+                return u'One'
 
             def transformIterable(self, result, encoding):
-                return "One"
+                return 'One'
 
-        class Transform2(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform2(FauxTransformBase):
+            pass
 
-            order = 1
-
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
-            def transformBytes(self, result, encoding):
-                return None
-
-            def transformUnicode(self, result, encoding):
-                return None
-
-            def transformIterable(self, result, encoding):
-                return None
-
-        class Transform3(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform3(FauxTransformBase):
 
             order = 2
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " three"
+                return result + b' three'
 
             def transformUnicode(self, result, encoding):
-                return result + u" three"
+                return result + u' three'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " three"
+                return ''.join(result) + ' three'
 
-        provideAdapter(Transform1, name=u"test.one")
-        provideAdapter(Transform2, name=u"test.two")
-        provideAdapter(Transform3, name=u"test.three")
+        provideAdapter(Transform1, name=u'test.one')
+        provideAdapter(Transform2, name=u'test.two')
+        provideAdapter(Transform3, name=u'test.three')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Blah"]
+        result = ['Blah']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("One three", new_result)
+        self.assertEqual('One three', new_result)
 
     def test_ordering(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform1(FauxTransformBase):
 
             order = 100
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " One"
+                return result + b' One'
 
             def transformUnicode(self, result, encoding):
-                return result + u" One"
+                return result + u' One'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " One"
+                return ''.join(result) + ' One'
 
-        class Transform2(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform2(FauxTransformBase):
 
             order = -100
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " Two"
+                return result + b' Two'
 
             def transformUnicode(self, result, encoding):
-                return result + u" Two"
+                return result + u' Two'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " Two"
+                return ''.join(result) + ' Two'
 
-        class Transform3(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform3(FauxTransformBase):
 
             order = 101
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " Three"
+                return result + b' Three'
 
             def transformUnicode(self, result, encoding):
-                return result + u" Three"
+                return result + u' Three'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " Three"
+                return ''.join(result) + ' Three'
 
-        provideAdapter(Transform1, name=u"test.one")
-        provideAdapter(Transform2, name=u"test.two")
-        provideAdapter(Transform3, name=u"test.three")
+        provideAdapter(Transform1, name=u'test.one')
+        provideAdapter(Transform2, name=u'test.two')
+        provideAdapter(Transform3, name=u'test.three')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Initial"]
+        result = ['Initial']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Initial Two One Three", new_result)
+        self.assertEqual('Initial Two One Three', new_result)
 
     def test_request_marker(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform1(FauxTransformBase):
 
             order = 100
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " One"
+                return result + b' One'
 
             def transformUnicode(self, result, encoding):
-                return result + u" One"
+                return result + u' One'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " One"
+                return ''.join(result) + ' One'
 
-        class Transform2(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform2(FauxTransformBase):
 
             order = -100
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " Two"
+                return result + b' Two'
 
             def transformUnicode(self, result, encoding):
-                return result + u" Two"
+                return result + u' Two'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " Two"
+                return ''.join(result) + ' Two'
 
-        class Transform3(object):
-            implements(ITransform)
-            adapts(Interface, IRequestMarker)
+        @implementer(ITransform)
+        @adapter(Interface, IRequestMarker)
+        class Transform3(FauxTransformBase):
 
             order = 101
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " Three"
+                return result + b' Three'
 
             def transformUnicode(self, result, encoding):
-                return result + u" Three"
+                return result + u' Three'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " Three"
+                return ''.join(result) + ' Three'
 
-        provideAdapter(Transform1, name=u"test.one")
-        provideAdapter(Transform2, name=u"test.two")
-        provideAdapter(Transform3, name=u"test.three")
+        provideAdapter(Transform1, name=u'test.one')
+        provideAdapter(Transform2, name=u'test.two')
+        provideAdapter(Transform3, name=u'test.three')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Initial"]
+        result = ['Initial']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Initial Two One", new_result)
+        self.assertEqual('Initial Two One', new_result)
 
         published = FauxPublished()
         request = FauxRequest(published)
         alsoProvides(request, IRequestMarker)
-        result = ["Initial"]
+        result = ['Initial']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Initial Two One Three", new_result)
+        self.assertEqual('Initial Two One Three', new_result)
 
     def test_published_marker(self):
 
-        class Transform1(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform1(FauxTransformBase):
 
             order = 100
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " One"
+                return result + b' One'
 
             def transformUnicode(self, result, encoding):
-                return result + u" One"
+                return result + u' One'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " One"
+                return ''.join(result) + ' One'
 
-        class Transform2(object):
-            implements(ITransform)
-            adapts(Interface, Interface)
+        class Transform2(FauxTransformBase):
 
             order = -100
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " Two"
+                return result + b' Two'
 
             def transformUnicode(self, result, encoding):
-                return result + u" Two"
+                return result + u' Two'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " Two"
+                return ''.join(result) + ' Two'
 
-        class Transform3(object):
-            implements(ITransform)
-            adapts(IPublishedMarker, Interface)
+        @implementer(ITransform)
+        @adapter(IPublishedMarker, Interface)
+        class Transform3(FauxTransformBase):
 
             order = 101
 
-            def __init__(self, published, request):
-                self.published = published
-                self.request = request
-
             def transformBytes(self, result, encoding):
-                return result + " Three"
+                return result + b' Three'
 
             def transformUnicode(self, result, encoding):
-                return result + u" Three"
+                return result + u' Three'
 
             def transformIterable(self, result, encoding):
-                return ''.join(result) + " Three"
+                return ''.join(result) + ' Three'
 
-        provideAdapter(Transform1, name=u"test.one")
-        provideAdapter(Transform2, name=u"test.two")
-        provideAdapter(Transform3, name=u"test.three")
+        provideAdapter(Transform1, name=u'test.one')
+        provideAdapter(Transform2, name=u'test.two')
+        provideAdapter(Transform3, name=u'test.three')
 
         published = FauxPublished()
         request = FauxRequest(published)
-        result = ["Initial"]
+        result = ['Initial']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Initial Two One", new_result)
+        self.assertEqual('Initial Two One', new_result)
 
         published = FauxPublished()
         alsoProvides(published, IPublishedMarker)
         request = FauxRequest(published)
-        result = ["Initial"]
+        result = ['Initial']
         encoding = 'utf-8'
 
         new_result = self.t(request, result, encoding)
-        self.assertEquals("Initial Two One Three", new_result)
+        self.assertEqual('Initial Two One Three', new_result)
+
 
 class TestZPublisherTransforms(unittest.TestCase):
 
@@ -641,11 +491,13 @@ class TestZPublisherTransforms(unittest.TestCase):
         self.t = Transformer()
 
     def test_applyTransform_webdav_port(self):
+
+        @implementer(ITransformer)
         class DoNotCallTransformer(object):
-            implements(ITransformer)
             encoding = None
+
             def __call__(self, request, result, encoding):
-                raise AssertionError("Shouldn't have been called")
+                raise AssertionError('Should not have been called')
 
         transformer = DoNotCallTransformer()
         provideUtility(transformer)
@@ -656,11 +508,12 @@ class TestZPublisherTransforms(unittest.TestCase):
         applyTransformOnSuccess(FauxPubEvent(request))
 
     def test_applyTransform_webdav_method(self):
+        @implementer(ITransformer)
         class DoNotCallTransformer(object):
-            implements(ITransformer)
             encoding = None
+
             def __call__(self, request, result, encoding):
-                raise AssertionError("Shouldn't have been called")
+                raise AssertionError('Should not have been called')
 
         transformer = DoNotCallTransformer()
         provideUtility(transformer)
@@ -671,11 +524,13 @@ class TestZPublisherTransforms(unittest.TestCase):
         applyTransformOnSuccess(FauxPubEvent(request))
 
     def test_applyTransform_webdav_pathinfo(self):
+
+        @implementer(ITransformer)
         class DoNotCallTransformer(object):
-            implements(ITransformer)
             encoding = None
+
             def __call__(self, request, result, encoding):
-                raise AssertionError("Shouldn't have been called")
+                raise AssertionError('Should not have been called')
 
         transformer = DoNotCallTransformer()
         provideUtility(transformer)
@@ -691,9 +546,11 @@ class TestZPublisherTransforms(unittest.TestCase):
         applyTransformOnSuccess(FauxPubEvent(request))
 
     def test_applyTransform_default_encoding(self):
+
+        @implementer(ITransformer)
         class EncodingCaptureTransformer(object):
-            implements(ITransformer)
             encoding = None
+
             def __call__(self, request, result, encoding):
                 self.encoding = encoding
 
@@ -704,12 +561,13 @@ class TestZPublisherTransforms(unittest.TestCase):
         request = FauxRequest(published)
         applyTransformOnSuccess(FauxPubEvent(request))
 
-        self.assertEquals(default_encoding, transformer.encoding)
+        self.assertEqual(default_encoding, transformer.encoding)
 
     def test_applyTransform_other_encoding(self):
+        @implementer(ITransformer)
         class EncodingCaptureTransformer(object):
-            implements(ITransformer)
             encoding = None
+
             def __call__(self, request, result, encoding):
                 self.encoding = encoding
 
@@ -718,14 +576,33 @@ class TestZPublisherTransforms(unittest.TestCase):
 
         published = FauxPublished()
         request = FauxRequest(published)
-        request.response.headers['content-type'] = 'text/html; charset=dummy'
+        request.response.headers['content-type'] = 'text/html; charset=utf-16'
         applyTransformOnSuccess(FauxPubEvent(request))
 
-        self.assertEquals("dummy", transformer.encoding)
+        self.assertEqual('utf-16', transformer.encoding)
+
+    def test_applyTransform_other_encoding_with_header_missing_space(self):
+        @implementer(ITransformer)
+        class EncodingCaptureTransformer(object):
+            encoding = None
+
+            def __call__(self, request, result, encoding):
+                self.encoding = encoding
+
+        transformer = EncodingCaptureTransformer()
+        provideUtility(transformer)
+
+        published = FauxPublished()
+        request = FauxRequest(published)
+        request.response.headers['content-type'] = 'text/html;charset=utf-16'
+        applyTransformOnSuccess(FauxPubEvent(request))
+
+        self.assertEqual('utf-16', transformer.encoding)
 
     def test_applyTransform_str(self):
+        @implementer(ITransformer)
         class FauxTransformer(object):
-            implements(ITransformer)
+
             def __call__(self, request, result, encoding):
                 return 'dummystr'
 
@@ -736,11 +613,12 @@ class TestZPublisherTransforms(unittest.TestCase):
         request = FauxRequest(published)
         applyTransformOnSuccess(FauxPubEvent(request))
 
-        self.assertEquals('dummystr', request.response.getBody())
+        self.assertEqual('dummystr', request.response.getBody())
 
     def test_applyTransform_unicode(self):
+        @implementer(ITransformer)
         class FauxTransformer(object):
-            implements(ITransformer)
+
             def __call__(self, request, result, encoding):
                 return u'dummystr'
 
@@ -752,11 +630,12 @@ class TestZPublisherTransforms(unittest.TestCase):
         applyTransformOnSuccess(FauxPubEvent(request))
 
         # note: the real setBody would encode here
-        self.assertEquals(u'dummystr', request.response.getBody())
+        self.assertEqual(u'dummystr', request.response.getBody())
 
     def test_applyTransform_iterable(self):
+        @implementer(ITransformer)
         class FauxTransformer(object):
-            implements(ITransformer)
+
             def __call__(self, request, result, encoding):
                 return ['iter', 'one']
 
@@ -767,18 +646,18 @@ class TestZPublisherTransforms(unittest.TestCase):
         request = FauxRequest(published)
         applyTransformOnSuccess(FauxPubEvent(request))
 
-        self.assertEquals('iterone', request.response.getBody())
+        self.assertEqual('iterone', request.response.getBody())
 
     def test_applyTransform_streamiterator(self):
         tmp = tempfile.mkstemp()[1]
         try:
 
-            out = open(tmp, 'w')
-            print >> out, "foo"
-            out.close()
+            with open(tmp, 'w') as out:
+                out.write('foo')
 
+            @implementer(ITransformer)
             class FauxTransformer(object):
-                implements(ITransformer)
+
                 def __call__(self, request, result, encoding):
                     return filestream_iterator(tmp)
 
@@ -789,16 +668,22 @@ class TestZPublisherTransforms(unittest.TestCase):
             request = FauxRequest(published)
             applyTransformOnSuccess(FauxPubEvent(request))
 
-            self.failUnless(isinstance(request.response.getBody(), filestream_iterator,))
+            self.assertTrue(
+                isinstance(
+                    request.response.getBody(),
+                    filestream_iterator
+                )
+            )
         finally:
             os.unlink(tmp)
 
     def test_applyTransform_str_input_body(self):
+        @implementer(ITransformer)
         class FauxTransformer(object):
-            implements(ITransformer)
+
             def __call__(self, request, result, encoding):
                 assert isinstance(result, list)
-                assert isinstance(result[0], str)
+                assert isinstance(result[0], six.binary_type)
                 return 'dummystr'
 
         transformer = FauxTransformer()
@@ -806,19 +691,20 @@ class TestZPublisherTransforms(unittest.TestCase):
 
         published = FauxPublished()
         request = FauxRequest(published)
-        request.response.setBody("<html />")
+        request.response.setBody('<html />')
 
         applyTransformOnSuccess(FauxPubEvent(request))
 
         # note: the real setBody would encode here
-        self.assertEquals('dummystr', request.response.getBody())
+        self.assertEqual('dummystr', request.response.getBody())
 
     def test_applyTransform_unicode_input_body(self):
+        @implementer(ITransformer)
         class FauxTransformer(object):
-            implements(ITransformer)
+
             def __call__(self, request, result, encoding):
                 assert isinstance(result, list)
-                assert isinstance(result[0], str)
+                assert isinstance(result[0], six.binary_type)
                 return u'dummystr'
 
         transformer = FauxTransformer()
@@ -826,9 +712,9 @@ class TestZPublisherTransforms(unittest.TestCase):
 
         published = FauxPublished()
         request = FauxRequest(published)
-        request.response.setBody(u"<html />")
+        request.response.setBody(u'<html />')
 
         applyTransformOnSuccess(FauxPubEvent(request))
 
         # note: the real setBody would encode here
-        self.assertEquals(u'dummystr', request.response.getBody())
+        self.assertEqual(u'dummystr', request.response.getBody())
